/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.jrunit.sample.decorated;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import org.jboss.jrunit.controller.ThreadLocalBenchmark;
import org.jboss.jrunit.decorators.ThreadLocalDecorator;

import junit.framework.Test;
import junit.framework.TestCase;

/**
 * This is a very simple client for calling upon the server via sockets and just send simple
 * text with the thread id of the client caller.<p>
 * What is important to note is that for each thread specified to the test decorator, there
 * will be a new instances of this class created, so none of the local resources will be shared.
 * However, for each loop specified to the test decorator, the same class instance will be used
 * when calling the test methods for the number of times specified by the loop parameter.  Therefore,
 * the local resources, such as socket and streams will be reused.<p>
 * Remember, this is just simple test, so resources are not being closed and cleaned up properly
 * (yes this is bad, but not what this tests is about... fix it if it bothers you that much).
 * <p/>
 * Another important point, which is specific to the server implementation but will be seen in the client,
 * is that the server processes each socket serially.  Meaning that there is only one thread on the server
 * to process request for each socket connection established.  The result of this is that you will see that
 * only thread will be processed at a time, so will process all the loops for a thread, then proceed to the next
 * thread and so forth.
 * <p/>
 *
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class SimpleClientTest extends TestCase
{
   private String address = "localhost";
   private int port = 6700;
   private Socket socket = null;
   private ObjectOutputStream oos;
   private ObjectInputStream objInputStream;

   public static Test suite()
   {
      return new ThreadLocalDecorator(SimpleClientTest.class, 3, 10, 0, true);
   }

   public void testClientCall() throws Exception
   {
      // start benchmark tracking
      ThreadLocalBenchmark.openBench("ClientCall");
      ThreadLocalBenchmark.openBench("GetSocket");

      getSocket();

      ThreadLocalBenchmark.closeBench("GetSocket");

      /**
       * Uncomment this to see that is the same object, thread, and socket for each loop,
       * but will be different for each thread as specified to the test decorator.  This
       * is based on the values passed to JunitThreadDecorator (or one of it's subclasses)
       * for the numberOfThreads and loop parameter values.
       */
      //System.out.println(hashCode() + " - " + Thread.currentThread().getName() + " - " + socket.hashCode());

      oos.writeObject("This is the request from " + Thread.currentThread().getName());
      oos.reset();
      oos.writeObject(Boolean.TRUE);
      oos.flush();
      oos.reset();

      Object obj = objInputStream.readObject();
      objInputStream.readObject();

      assertEquals("This is response.", obj);

      ThreadLocalBenchmark.closeBench("ClientCall");

   }

   public void testFailure()
   {
      fail("This is a check to make sure failures reported.");
   }

   private void getSocket() throws IOException
   {
      if(socket == null)
      {
         try
         {
            socket = new Socket(address, port);
            BufferedOutputStream out = new BufferedOutputStream(socket.getOutputStream());
            BufferedInputStream in = new BufferedInputStream(socket.getInputStream());

            oos = new ObjectOutputStream(out);
            objInputStream = new ObjectInputStream(in);
         }
         catch(IOException e)
         {
            e.printStackTrace();
         }
      }
      else
      {
         oos.reset();
         oos.writeByte(1);
         oos.flush();
         oos.reset();
         objInputStream.readByte();
      }
   }
}