/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.microbench.sample;

import org.jboss.jrunit.decorators.ThreadLocalDecorator;

import junit.framework.Test;
import junit.framework.TestCase;

/**
 * This is just a very simple test case to illustrate how the values for number
 * of threads and loops specified to the test decorator affect a test run.
 * <p>
 * ThreadLocalDecorator (and all the other decorators that accept number of threads and loops) -
 * for each thread specified in the number of threads, there will be a new instance of the test created.
 * For the number of loops specified, the test methods will be called on each test instance.
 * So for example, if specify 3 threads and 10 loops, there will be three instances of the
 * test class created and each instance will have their test methods called 10 times by
 * each of their respective threads.  This test class illustrates this.
 * <p>
 * In this particular case, the number of threads is set to 3 and the number of loops is
 * set to 10.  There are three variables defined for the class, two static and one local.
 * The local variable, <i>localCounter</i> will be incremented within the test method and will
 * see that there are three seperate instance of this variable (since there are three seperate instances
 * of the test class, each with their own local variable).  This local variable will be incremented for
 * each loop of each thread.  When the test is finished running the value for each of the instances will
 * be 10 (for 10 loops per the three instances)<p>
 * The two static variables are seen by all the instances created, so for each of the three test class
 * instances created.  The <i>staticCounter</i> is incremented in the constructor of this test class.
 * Therefore will see that are three instances created as when the test is run, the output for this variable
 * will be 3.  The <i>staticMethodCounter</i> is incremented within the test method, so this will be incremented
 * for each call (so per thread and per loop).  Therefore the value at end of test run will be 30
 * (3 threads X 10 loops).
 * <p>
 * So when test is run, should see following as last line of test output:<p>
 * staticCounter = 3 <p>
 * staticMethodcounter = 30 <p>
 * localCounter = 10 <p>
 * <p>
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class SimpleThreadLoopCounter extends TestCase
{
   private static int staticCounter = 0;
   private static int staticMethodCounter = 0;
   private int localCounter = 0;

   public static Test suite()
   {
      return new ThreadLocalDecorator(SimpleThreadLoopCounter.class, 3, 10, 0, true);
   }

   public SimpleThreadLoopCounter()
   {
      staticCounter++;
   }

   public void testCounter() throws Exception
   {
      System.out.println("staticCounter = " + staticCounter);
      System.out.println("staticMethodcounter = " + ++staticMethodCounter);
      System.out.println("localCounter = " + ++localCounter);
   }

   /**
    * Another issue in regards to the way junit handles test clases with multiple test methods
    * is to create a new instance of the test class for each test method it is going to run.
    * So if uncomment the following test method and ran this test class again, would see the
    * following output as the last lines:
    * staticCounter = 6
    * staticMethodcounter = 60
    * localCounter = 10
    */
//   public void testCounter2() throws Exception
//   {
//      System.out.println("staticCounter = " + staticCounter);
//      System.out.println("staticMethodcounter = " + ++staticMethodCounter);
//      System.out.println("localCounter = " + ++localCounter);
//   }

}